/**
 * Shared constants for the CLI
 */

// Indentation constants
export const INDENT_EVALUATOR = 6;
export const INDENT_AGENT = 2;

// Panel offsets
export const OFFSET_PLAN = 3;
export const OFFSET_TASK = 3;
export const OFFSET_SUMMARY = 7;
export const OFFSET_RESUME = 1;
